/*============================================================================
 * This function is called at the end of each time step, and has a very
 *  general purpose
 *  (i.e. anything that does not have another dedicated user function)
 *============================================================================*/

/* code_saturne version 7.1 */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2021 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 * Local headers
 *----------------------------------------------------------------------------*/

#include "cs_headers.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_extra_operations.c
 *
 * \brief This function is called at the end of each time step, and has a very
 * general purpose (i.e. anything that does not have another dedicated
 * user function)
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*Ce fichier d'utilisateur permet de calculer la température moyenne des parois de la barre*/

void
cs_user_extra_operations(cs_domain_t     *domain)
{
  // Champ de température dans toutes les cellules: c'est en tableau
  const cs_field_t *f = CS_F_(t_b);

  // Prendre des faces des barres
  const cs_zone_t *zone1 = cs_boundary_zone_by_name_try("Barre1_P1");

  // Définir une variable pour faire la somme de la température
  cs_real_t somme1 = 0;

  // Récupérer le champ de température de chaque point
  for (cs_lnum_t ilelt = 0; ilelt < zone1->n_elts; ilelt++) 
   {
    cs_lnum_t face_id = zone1->elt_ids[ilelt];
    somme1 = somme1 + f->val[face_id];
   }

  // Calculer la moyenne de température
  cs_real_t t_moyenne1 = somme1 / (zone1->n_elts);

  // Ecrire la valeur dans le fichier Notebook
  cs_notebook_parameter_set_value("Temperature_B1P1",t_moyenne1);

  // Extraire tous les valeurs de température moyenne de chaque pas de temps en format csv
  FILE *file = fopen("Temperature_barre.csv","a");
  fprintf(file,"Time step; %f; Temperature_B1P1; %f\n",cs_glob_time_step->t_cur, t_moyenne1);
  fclose(file);

}


/*----------------------------------------------------------------------------*/

END_C_DECLS
